@@ -145,6 +145,13 @@ static NTSTATUS netsec_do_seq_num(struct schannel_state *state,
 				  uint32_t checksum_length,
 				  uint8_t seq_num[8])
 {
+	if (state->creds->authenticate_kerberos) {
+		DBG_WARNING("Called with authenticate_kerberos from %s %s\n",
+			    state->creds->account_name,
+			    state->creds->computer_name);
+		return NT_STATUS_ACCESS_DENIED;
+	}
+
 	if (state->creds->negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
 		gnutls_cipher_hd_t cipher_hnd = NULL;
 		gnutls_datum_t key = {
@@ -208,12 +215,11 @@ static NTSTATUS netsec_do_seq_num(struct schannel_state *state,
 				      checksum,
 				      checksum_length,
 				      _sequence_key);
+		ZERO_ARRAY(digest1);
 		if (rc < 0) {
 			return gnutls_error_to_ntstatus(rc, NT_STATUS_HMAC_NOT_SUPPORTED);
 		}
 
-		ZERO_ARRAY(digest1);
-
 		rc = gnutls_cipher_init(&cipher_hnd,
 					GNUTLS_CIPHER_ARCFOUR_128,
 					&sequence_key,
@@ -244,6 +250,13 @@ static NTSTATUS netsec_do_seal(struct schannel_state *state,
 			       uint8_t *data, uint32_t length,
 			       bool forward)
 {
+	if (state->creds->authenticate_kerberos) {
+		DBG_WARNING("Called with authenticate_kerberos from %s %s\n",
+			    state->creds->account_name,
+			    state->creds->computer_name);
+		return NT_STATUS_ACCESS_DENIED;
+	}
+
 	if (state->creds->negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
 		gnutls_cipher_hd_t cipher_hnd = NULL;
 		uint8_t sess_kf0[16] = {0};
@@ -424,6 +437,13 @@ static NTSTATUS netsec_do_sign(struct schannel_state *state,
 			       uint8_t header[8],
 			       uint8_t *checksum)
 {
+	if (state->creds->authenticate_kerberos) {
+		DBG_WARNING("Called with authenticate_kerberos from %s %s\n",
+			    state->creds->account_name,
+			    state->creds->computer_name);
+		return NT_STATUS_ACCESS_DENIED;
+	}
+
 	if (state->creds->negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
 		gnutls_hmac_hd_t hmac_hnd = NULL;
 		int rc;
@@ -480,13 +500,13 @@ static NTSTATUS netsec_do_sign(struct schannel_state *state,
 
 		rc = gnutls_hash_init(&hash_hnd, GNUTLS_DIG_MD5);
 		if (rc < 0) {
-			return gnutls_error_to_ntstatus(rc, NT_STATUS_HMAC_NOT_SUPPORTED);
+			return gnutls_error_to_ntstatus(rc, NT_STATUS_HASH_NOT_SUPPORTED);
 		}
 
 		rc = gnutls_hash(hash_hnd, zeros, sizeof(zeros));
 		if (rc < 0) {
 			gnutls_hash_deinit(hash_hnd, NULL);
-			return gnutls_error_to_ntstatus(rc, NT_STATUS_HMAC_NOT_SUPPORTED);
+			return gnutls_error_to_ntstatus(rc, NT_STATUS_HASH_NOT_SUPPORTED);
 		}
 		if (confounder) {
 			SSVAL(header, 0, NL_SIGN_HMAC_MD5);
@@ -497,12 +517,12 @@ static NTSTATUS netsec_do_sign(struct schannel_state *state,
 			rc = gnutls_hash(hash_hnd, header, 8);
 			if (rc < 0) {
 				gnutls_hash_deinit(hash_hnd, NULL);
-				return gnutls_error_to_ntstatus(rc, NT_STATUS_HMAC_NOT_SUPPORTED);
+				return gnutls_error_to_ntstatus(rc, NT_STATUS_HASH_NOT_SUPPORTED);
 			}
 			rc = gnutls_hash(hash_hnd, confounder, 8);
 			if (rc < 0) {
 				gnutls_hash_deinit(hash_hnd, NULL);
-				return gnutls_error_to_ntstatus(rc, NT_STATUS_HMAC_NOT_SUPPORTED);
+				return gnutls_error_to_ntstatus(rc, NT_STATUS_HASH_NOT_SUPPORTED);
 			}
 		} else {
 			SSVAL(header, 0, NL_SIGN_HMAC_MD5);
@@ -513,13 +533,13 @@ static NTSTATUS netsec_do_sign(struct schannel_state *state,
 			rc = gnutls_hash(hash_hnd, header, 8);
 			if (rc < 0) {
 				gnutls_hash_deinit(hash_hnd, NULL);
-				return gnutls_error_to_ntstatus(rc, NT_STATUS_HMAC_NOT_SUPPORTED);
+				return gnutls_error_to_ntstatus(rc, NT_STATUS_HASH_NOT_SUPPORTED);
 			}
 		}
 		rc = gnutls_hash(hash_hnd, data, length);
 		if (rc < 0) {
 			gnutls_hash_deinit(hash_hnd, NULL);
-			return gnutls_error_to_ntstatus(rc, NT_STATUS_HMAC_NOT_SUPPORTED);
+			return gnutls_error_to_ntstatus(rc, NT_STATUS_HASH_NOT_SUPPORTED);
 		}
 		gnutls_hash_deinit(hash_hnd, packet_digest);
 
@@ -834,6 +854,16 @@ static NTSTATUS schannel_update_internal(struct gensec_security *gensec_security
 			return NT_STATUS_INVALID_PARAMETER_MIX;
 		}
 
+		if (creds->authenticate_kerberos) {
+			DBG_ERR("attempted schannel connection with "
+				"authenticate_kerberos from %s %s\n",
+				creds->account_name,
+				creds->computer_name);
+			NDR_PRINT_DEBUG(netlogon_creds_CredentialState, creds);
+			log_stack_trace();
+			return NT_STATUS_INVALID_PARAMETER_MIX;
+		}
+
 		state = netsec_create_state(gensec_security,
 					    creds, true /* initiator */);
 		if (state == NULL) {
