@@ -25,6 +25,8 @@ import ldb
 import socket
 import samba
 import re
+import os
+import tempfile
 from samba import sd_utils
 from samba.dcerpc import dnsserver, dnsp, security
 from samba.dnsserver import ARecord, AAAARecord
@@ -32,6 +34,9 @@ from samba.ndr import ndr_unpack, ndr_pack, ndr_print
 from samba.remove_dc import remove_dns_references
 from samba.auth import system_session
 from samba.samdb import SamDB
+from samba.common import get_bytes
+from subprocess import check_call, CalledProcessError
+from . import common
 
 from samba import (
     credentials,
@@ -48,7 +53,6 @@ from samba.netcmd import (
     Option,
 )
 
-
 def _is_valid_ip(ip_string, address_families=None):
     """Check ip string is valid address"""
     # by default, check both ipv4 and ipv6
@@ -101,7 +105,6 @@ def add_dns_records(
     except WERRORError as e:
         if e.args[0] == werror.WERR_DNS_ERROR_NAME_DOES_NOT_EXIST:
             name_found = False
-            pass
 
     if name_found:
         for rec in res.rec:
@@ -169,36 +172,36 @@ def add_dns_records(
         )
 
 
-class cmd_computer_create(Command):
-    """Create a new computer.
+class cmd_computer_add(Command):
+    """Add a new computer.
 
-This command creates a new computer account in the Active Directory domain.
+This command adds a new computer account to the Active Directory domain.
 The computername specified on the command is the sAMaccountName without the
 trailing $ (dollar sign).
 
-User accounts may represent physical entities, such as workstations. Computer
+Computer accounts may represent physical entities, such as workstations. Computer
 accounts are also referred to as security principals and are assigned a
 security identifier (SID).
 
 Example1:
-samba-tool computer create Computer1 -H ldap://samba.samdom.example.com \
+samba-tool computer add Computer1 -H ldap://samba.samdom.example.com \\
     -Uadministrator%passw1rd
 
-Example1 shows how to create a new computer in the domain against a remote LDAP
+Example1 shows how to add a new computer to the domain against a remote LDAP
 server. The -H parameter is used to specify the remote target server. The -U
 option is used to pass the userid and password authorized to issue the command
 remotely.
 
 Example2:
-sudo samba-tool computer create Computer2
+sudo samba-tool computer add Computer2
 
-Example2 shows how to create a new computer in the domain against the local
+Example2 shows how to add a new computer to the domain against the local
 server. sudo is used so a user may run the command as root.
 
 Example3:
-samba-tool computer create Computer3 --computerou='OU=OrgUnit'
+samba-tool computer add Computer3 --computerou='OU=OrgUnit'
 
-Example3 shows how to create a new computer in the OrgUnit organizational unit.
+Example3 shows how to add a new computer in the OrgUnit organizational unit.
 
 """
     synopsis = "%prog <computername> [options]"
@@ -211,7 +214,7 @@ Example3 shows how to create a new computer in the OrgUnit organizational unit.
                      "counterpart) to default CN=Computers in which new "
                      "computer object will be created. E.g. 'OU=<OU name>'"),
                type=str),
-        Option("--description", help="Computers's description", type=str),
+        Option("--description", help="Computer's description", type=str),
         Option("--prepare-oldjoin",
                help="Prepare enabled machine account for oldjoin mechanism",
                action="store_true"),
@@ -299,10 +302,10 @@ Example3 shows how to create a new computer in the OrgUnit organizational unit.
                     change_owner_sd, samdb.host_dns_name(),
                     ip_address_list, self.get_logger())
         except Exception as e:
-            raise CommandError("Failed to create computer '%s': " %
+            raise CommandError("Failed to add computer '%s': " %
                                computername, e)
 
-        self.outf.write("Computer '%s' created successfully\n" % computername)
+        self.outf.write("Computer '%s' added successfully\n" % computername)
 
 
 class cmd_computer_delete(Command):
@@ -323,7 +326,7 @@ userid. The -H or --URL= option can be used to execute the command against
 a remote server.
 
 Example1:
-samba-tool computer delete Computer1 -H ldap://samba.samdom.example.com \
+samba-tool computer delete Computer1 -H ldap://samba.samdom.example.com \\
     -Uadministrator%passw1rd
 
 Example1 shows how to delete a computer in the domain against a remote LDAP
@@ -400,6 +403,121 @@ sudo is used so a computer may run the command as root.
         self.outf.write("Deleted computer %s\n" % computername)
 
 
+class cmd_computer_edit(Command):
+    """Modify Computer AD object.
+
+    This command will allow editing of a computer account in the Active
+    Directory domain. You will then be able to add or change attributes and
+    their values.
+
+    The computername specified on the command is the sAMaccountName with or
+    without the trailing $ (dollar sign).
+
+    The command may be run from the root userid or another authorized userid.
+
+    The -H or --URL= option can be used to execute the command against a remote
+    server.
+
+    Example1:
+    samba-tool computer edit Computer1 -H ldap://samba.samdom.example.com \\
+        -U administrator --password=passw1rd
+
+    Example1 shows how to edit a computers attributes in the domain against a
+    remote LDAP server.
+
+    The -H parameter is used to specify the remote target server.
+
+    Example2:
+    samba-tool computer edit Computer2
+
+    Example2 shows how to edit a computers attributes in the domain against a
+    local LDAP server.
+
+    Example3:
+    samba-tool computer edit Computer3 --editor=nano
+
+    Example3 shows how to edit a computers attributes in the domain against a
+    local LDAP server using the 'nano' editor.
+    """
+    synopsis = "%prog <computername> [options]"
+
+    takes_options = [
+        Option("-H", "--URL", help="LDB URL for database or target server",
+               type=str, metavar="URL", dest="H"),
+        Option("--editor", help="Editor to use instead of the system default,"
+               " or 'vi' if no system default is set.", type=str),
+    ]
+
+    takes_args = ["computername"]
+    takes_optiongroups = {
+        "sambaopts": options.SambaOptions,
+        "credopts": options.CredentialsOptions,
+        "versionopts": options.VersionOptions,
+    }
+
+    def run(self, computername, credopts=None, sambaopts=None, versionopts=None,
+            H=None, editor=None):
+        lp = sambaopts.get_loadparm()
+        creds = credopts.get_credentials(lp, fallback_machine=True)
+        samdb = SamDB(url=H, session_info=system_session(),
+                      credentials=creds, lp=lp)
+
+        samaccountname = computername
+        if not computername.endswith('$'):
+            samaccountname = "%s$" % computername
+
+        filter = ("(&(sAMAccountType=%d)(sAMAccountName=%s))" %
+                  (dsdb.ATYPE_WORKSTATION_TRUST,
+                   ldb.binary_encode(samaccountname)))
+
+        domaindn = samdb.domain_dn()
+
+        try:
+            res = samdb.search(base=domaindn,
+                               expression=filter,
+                               scope=ldb.SCOPE_SUBTREE)
+            computer_dn = res[0].dn
+        except IndexError:
+            raise CommandError('Unable to find computer "%s"' % (computername))
+
+        if len(res) != 1:
+            raise CommandError('Invalid number of results: for "%s": %d' %
+                               ((computername), len(res)))
+
+        msg = res[0]
+        result_ldif = common.get_ldif_for_editor(samdb, msg)
+
+        if editor is None:
+            editor = os.environ.get('EDITOR')
+            if editor is None:
+                editor = 'vi'
+
+        with tempfile.NamedTemporaryFile(suffix=".tmp") as t_file:
+            t_file.write(get_bytes(result_ldif))
+            t_file.flush()
+            try:
+                check_call([editor, t_file.name])
+            except CalledProcessError as e:
+                raise CalledProcessError("ERROR: ", e)
+            with open(t_file.name) as edited_file:
+                edited_message = edited_file.read()
+
+        msgs_edited = samdb.parse_ldif(edited_message)
+        msg_edited = next(msgs_edited)[1]
+
+        res_msg_diff = samdb.msg_diff(msg, msg_edited)
+        if len(res_msg_diff) == 0:
+            self.outf.write("Nothing to do\n")
+            return
+
+        try:
+            samdb.modify(res_msg_diff)
+        except Exception as e:
+            raise CommandError("Failed to modify computer '%s': " %
+                               computername, e)
+
+        self.outf.write("Modified computer '%s' successfully\n" % computername)
+
 class cmd_computer_list(Command):
     """List all computers."""
 
@@ -408,6 +526,13 @@ class cmd_computer_list(Command):
     takes_options = [
         Option("-H", "--URL", help="LDB URL for database or target server",
                type=str, metavar="URL", dest="H"),
+        Option("-b", "--base-dn",
+               help="Specify base DN to use",
+               type=str),
+        Option("--full-dn", dest="full_dn",
+               default=False,
+               action="store_true",
+               help="Display DN instead of the sAMAccountName.")
     ]
 
     takes_optiongroups = {
@@ -416,7 +541,13 @@ class cmd_computer_list(Command):
         "versionopts": options.VersionOptions,
     }
 
-    def run(self, sambaopts=None, credopts=None, versionopts=None, H=None):
+    def run(self,
+            sambaopts=None,
+            credopts=None,
+            versionopts=None,
+            H=None,
+            base_dn=None,
+            full_dn=False):
         lp = sambaopts.get_loadparm()
         creds = credopts.get_credentials(lp, fallback_machine=True)
 
@@ -425,14 +556,22 @@ class cmd_computer_list(Command):
 
         filter = "(sAMAccountType=%u)" % (dsdb.ATYPE_WORKSTATION_TRUST)
 
-        domain_dn = samdb.domain_dn()
-        res = samdb.search(domain_dn, scope=ldb.SCOPE_SUBTREE,
+        search_dn = samdb.domain_dn()
+        if base_dn:
+            search_dn = samdb.normalize_dn_in_domain(base_dn)
+
+        res = samdb.search(search_dn,
+                           scope=ldb.SCOPE_SUBTREE,
                            expression=filter,
                            attrs=["samaccountname"])
         if (len(res) == 0):
             return
 
         for msg in res:
+            if full_dn:
+                self.outf.write("%s\n" % msg.get("dn"))
+                continue
+
             self.outf.write("%s\n" % msg.get("samaccountname", idx=0))
 
 
@@ -450,7 +589,7 @@ The -H or --URL= option can be used to execute the command against a remote
 server.
 
 Example1:
-samba-tool computer show Computer1 -H ldap://samba.samdom.example.com \
+samba-tool computer show Computer1 -H ldap://samba.samdom.example.com \\
     -U administrator
 
 Example1 shows how display a computers attributes in the domain against a
@@ -519,14 +658,14 @@ attribute.
                                samaccountname)
 
         for msg in res:
-            computer_ldif = samdb.write_ldif(msg, ldb.CHANGETYPE_NONE)
+            computer_ldif = common.get_ldif_for_editor(samdb, msg)
             self.outf.write(computer_ldif)
 
 
 class cmd_computer_move(Command):
     """Move a computer to an organizational unit/container."""
 
-    synopsis = "%prog computername <new_ou_dn> [options]"
+    synopsis = "%prog <computername> <new_ou_dn> [options]"
 
     takes_options = [
         Option("-H", "--URL", help="LDB URL for database or target server",
@@ -581,8 +720,10 @@ class cmd_computer(SuperCommand):
     """Computer management."""
 
     subcommands = {}
-    subcommands["create"] = cmd_computer_create()
+    subcommands["add"] = cmd_computer_add()
+    subcommands["create"] = cmd_computer_add()
     subcommands["delete"] = cmd_computer_delete()
+    subcommands["edit"] = cmd_computer_edit()
     subcommands["list"] = cmd_computer_list()
     subcommands["show"] = cmd_computer_show()
     subcommands["move"] = cmd_computer_move()
