@@ -18,11 +18,11 @@
  */
 
 #include "includes.h"
-#include "auth_info.h"
 
 #include "lib/netapi/netapi.h"
 #include "lib/netapi/netapi_private.h"
-#include "libsmb/libsmb.h"
+#include "source3/include/client.h"
+#include "source3/libsmb/proto.h"
 #include "rpc_client/cli_pipe.h"
 #include "../libcli/smb/smbXcli_base.h"
 
@@ -67,11 +67,16 @@ static WERROR libnetapi_open_ipc_connection(struct libnetapi_ctx *ctx,
 					    struct client_ipc_connection **pp)
 {
 	struct libnetapi_private_ctx *priv_ctx;
-	struct user_auth_info *auth_info = NULL;
 	struct cli_state *cli_ipc = NULL;
 	struct client_ipc_connection *p;
 	NTSTATUS status;
-	struct cli_credentials *creds = NULL;
+	const char *username = NULL;
+	const char *password = NULL;
+	NET_API_STATUS rc;
+	enum credentials_use_kerberos krb5_state;
+	struct smb_transports ts =
+		smb_transports_parse("client smb transports",
+				     lp_client_smb_transports());
 
 	if (!ctx || !pp || !server_name) {
 		return WERR_INVALID_PARAMETER;
@@ -85,38 +90,37 @@ static WERROR libnetapi_open_ipc_connection(struct libnetapi_ctx *ctx,
 		return WERR_OK;
 	}
 
-	auth_info = user_auth_info_init(ctx);
-	if (!auth_info) {
-		return WERR_NOT_ENOUGH_MEMORY;
-	}
-	set_cmdline_auth_info_signing_state_raw(auth_info, SMB_SIGNING_IPC_DEFAULT);
-	set_cmdline_auth_info_use_kerberos(auth_info, ctx->use_kerberos);
-	set_cmdline_auth_info_username(auth_info, ctx->username);
-	if (ctx->password) {
-		set_cmdline_auth_info_password(auth_info, ctx->password);
-	} else {
-		set_cmdline_auth_info_getpass(auth_info);
+	rc = libnetapi_get_username(ctx, &username);
+	if (rc != 0) {
+		return WERR_INTERNAL_ERROR;
 	}
 
-	if (ctx->username && ctx->username[0] &&
-	    ctx->password && ctx->password[0] &&
-	    ctx->use_kerberos) {
-		set_cmdline_auth_info_fallback_after_kerberos(auth_info, true);
+	rc = libnetapi_get_password(ctx, &password);
+	if (rc != 0) {
+		return WERR_INTERNAL_ERROR;
 	}
 
-	if (ctx->use_ccache) {
-		set_cmdline_auth_info_use_ccache(auth_info, true);
+	if (password == NULL) {
+		cli_credentials_set_cmdline_callbacks(ctx->creds);
+	}
+
+	krb5_state = cli_credentials_get_kerberos_state(ctx->creds);
+
+	if (username != NULL && username[0] != '\0' &&
+	    password != NULL && password[0] != '\0' &&
+	    krb5_state == CRED_USE_KERBEROS_REQUIRED) {
+		cli_credentials_set_kerberos_state(ctx->creds,
+						   CRED_USE_KERBEROS_DESIRED,
+						   CRED_SPECIFIED);
 	}
-	creds = get_cmdline_auth_info_creds(auth_info);
 
 	status = cli_cm_open(ctx, NULL,
 			     server_name, "IPC$",
-			     creds,
-			     NULL, 0, 0x20, &cli_ipc);
+			     ctx->creds,
+			     NULL, &ts, 0x20, &cli_ipc);
 	if (!NT_STATUS_IS_OK(status)) {
 		cli_ipc = NULL;
 	}
-	TALLOC_FREE(auth_info);
 
 	if (!cli_ipc) {
 		libnetapi_set_error_string(ctx,
@@ -163,7 +167,10 @@ static NTSTATUS pipe_cm_find(struct client_ipc_connection *ipc,
 	struct client_pipe_connection *p;
 
 	for (p = ipc->pipe_connections; p; p = p->next) {
+		struct dcerpc_binding_handle *bh = NULL;
+		const struct dcerpc_binding *bd = NULL;
 		const char *ipc_remote_name;
+		struct ndr_syntax_id syntax;
 
 		if (!rpccli_is_connected(p->pipe)) {
 			return NT_STATUS_PIPE_EMPTY;
@@ -171,9 +178,15 @@ static NTSTATUS pipe_cm_find(struct client_ipc_connection *ipc,
 
 		ipc_remote_name = smbXcli_conn_remote_name(ipc->cli->conn);
 
-		if (strequal(ipc_remote_name, p->pipe->desthost)
-		    && ndr_syntax_id_equal(&p->pipe->abstract_syntax,
-					   &table->syntax_id)) {
+		if (!strequal(ipc_remote_name, p->pipe->desthost)) {
+			continue;
+		}
+
+		bh = p->pipe->binding_handle;
+		bd = dcerpc_binding_handle_get_binding(bh);
+		syntax = dcerpc_binding_get_abstract_syntax(bd);
+
+		if (ndr_syntax_id_equal(&syntax, &table->syntax_id)) {
 			*presult = p->pipe;
 			return NT_STATUS_OK;
 		}
