@@ -20,6 +20,8 @@
 
 #include "includes.h"
 #include "librpc/rpc/dcerpc.h"
+#include "librpc/rpc/dcerpc_internal.h"
+#include "librpc/rpc/dcerpc_util.h"
 #include "librpc/gen_ndr/ndr_dcerpc.h"
 #include "librpc/crypto/gse.h"
 #include "auth/gensec/gensec.h"
@@ -32,7 +34,7 @@
 *
 * @param mem_ctx	The memory context the blob will be allocated on
 * @param ptype		The DCERPC packet type
-* @param pfc_flags	The DCERPC PFC Falgs
+* @param pfc_flags	The DCERPC PFC Flags
 * @param auth_length	The length of the trailing auth blob
 * @param call_id	The call ID
 * @param u		The payload of the packet
@@ -81,55 +83,6 @@ NTSTATUS dcerpc_push_ncacn_packet(TALLOC_CTX *mem_ctx,
 	return NT_STATUS_OK;
 }
 
-/**
-* @brief Decodes a ncacn_packet
-*
-* @param mem_ctx	The memory context on which to allocate the packet
-*			elements
-* @param blob		The blob of data to decode
-* @param r		An empty ncacn_packet, must not be NULL
-*
-* @return a NTSTATUS error code
-*/
-NTSTATUS dcerpc_pull_ncacn_packet(TALLOC_CTX *mem_ctx,
-				  const DATA_BLOB *blob,
-				  struct ncacn_packet *r)
-{
-	enum ndr_err_code ndr_err;
-	struct ndr_pull *ndr;
-
-	ndr = ndr_pull_init_blob(blob, mem_ctx);
-	if (!ndr) {
-		return NT_STATUS_NO_MEMORY;
-	}
-
-	if (!(CVAL(ndr->data, DCERPC_DREP_OFFSET) & DCERPC_DREP_LE)) {
-		ndr->flags |= LIBNDR_FLAG_BIGENDIAN;
-	}
-
-	if (CVAL(ndr->data, DCERPC_PFC_OFFSET) & DCERPC_PFC_FLAG_OBJECT_UUID) {
-		ndr->flags |= LIBNDR_FLAG_OBJECT_PRESENT;
-	}
-
-	ndr_err = ndr_pull_ncacn_packet(ndr, NDR_SCALARS|NDR_BUFFERS, r);
-
-	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
-		talloc_free(ndr);
-		return ndr_map_error2ntstatus(ndr_err);
-	}
-	talloc_free(ndr);
-
-	if (DEBUGLEVEL >= 10) {
-		NDR_PRINT_DEBUG(ncacn_packet, r);
-	}
-
-	if (r->frag_length != blob->length) {
-		return NT_STATUS_RPC_PROTOCOL_ERROR;
-	}
-
-	return NT_STATUS_OK;
-}
-
 /**
 * @brief NDR Encodes a dcerpc_auth structure
 *
@@ -204,7 +157,6 @@ NTSTATUS dcerpc_guess_sizes(struct pipe_auth_data *auth,
 	switch (auth->auth_level) {
 	case DCERPC_AUTH_LEVEL_NONE:
 	case DCERPC_AUTH_LEVEL_CONNECT:
-	case DCERPC_AUTH_LEVEL_PACKET:
 		max_len = max_xmit_frag - header_len;
 		*data_to_send = MIN(max_len, data_left);
 		*pad_len = 0;
@@ -218,6 +170,9 @@ NTSTATUS dcerpc_guess_sizes(struct pipe_auth_data *auth,
 	case DCERPC_AUTH_LEVEL_INTEGRITY:
 		break;
 
+	case DCERPC_AUTH_LEVEL_PACKET:
+		break;
+
 	default:
 		return NT_STATUS_INVALID_PARAMETER;
 	}
@@ -293,6 +248,7 @@ static NTSTATUS add_generic_auth_footer(struct gensec_security *gensec_security,
 		break;
 
 	case DCERPC_AUTH_LEVEL_INTEGRITY:
+	case DCERPC_AUTH_LEVEL_PACKET:
 		/* Data is signed. */
 		status = gensec_sign_packet(gensec_security,
 					    rpc_out->data,
@@ -350,6 +306,7 @@ static NTSTATUS get_generic_auth_footer(struct gensec_security *gensec_security,
 					    auth_token);
 
 	case DCERPC_AUTH_LEVEL_INTEGRITY:
+	case DCERPC_AUTH_LEVEL_PACKET:
 		/* Data is signed. */
 		return gensec_check_packet(gensec_security,
 					   data->data,
@@ -476,6 +433,10 @@ NTSTATUS dcerpc_check_auth(struct pipe_auth_data *auth,
 		DEBUG(10, ("Requested Integrity.\n"));
 		break;
 
+	case DCERPC_AUTH_LEVEL_PACKET:
+		DEBUG(10, ("Requested packet.\n"));
+		break;
+
 	case DCERPC_AUTH_LEVEL_CONNECT:
 		if (pkt->auth_length != 0) {
 			break;
@@ -491,7 +452,7 @@ NTSTATUS dcerpc_check_auth(struct pipe_auth_data *auth,
 		return NT_STATUS_OK;
 
 	default:
-		DEBUG(3, ("Unimplemented Auth Level %d",
+		DEBUG(3, ("Unimplemented Auth Level %d\n",
 			  auth->auth_level));
 		return NT_STATUS_INVALID_PARAMETER;
 	}
