@@ -1,4 +1,4 @@
-/* 
+/*
  * Fake ACLs VFS module.  Implements passthrough operation of all VFS
  * calls to disk functions, except for file ownership and ACLs, which
  * are stored in xattrs.
@@ -11,12 +11,12 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
- *  
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *  
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
@@ -126,12 +126,20 @@ static int fake_acls_stat(vfs_handle_struct *handle,
 			return -1;
 		}
 
-		/* Recursion guard. */
-		prd->calling_pathref_fsp = true;
-		status = openat_pathref_fsp(handle->conn->cwd_fsp,
-					    smb_fname_cp);
-		/* End recursion guard. */
-		prd->calling_pathref_fsp = false;
+		if (fsp_get_pathref_fd(handle->conn->cwd_fsp) == -1) {
+			/*
+			 * No tcon around, fail as if we don't have
+			 * the EAs
+			 */
+			status = NT_STATUS_INVALID_HANDLE;
+		} else {
+			/* Recursion guard. */
+			prd->calling_pathref_fsp = true;
+			status = openat_pathref_fsp(handle->conn->cwd_fsp,
+						    smb_fname_cp);
+			/* End recursion guard. */
+			prd->calling_pathref_fsp = false;
+		}
 
 		if (!NT_STATUS_IS_OK(status)) {
 			/*
@@ -269,7 +277,7 @@ static SMB_ACL_T fake_acls_blob2acl(DATA_BLOB *blob, TALLOC_CTX *mem_ctx)
 		return NULL;
 	}
 
-	ndr_err = ndr_pull_struct_blob(blob, acl, acl, 
+	ndr_err = ndr_pull_struct_blob(blob, acl, acl,
 		(ndr_pull_flags_fn_t)ndr_pull_smb_acl_t);
 
 	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
@@ -285,7 +293,7 @@ static DATA_BLOB fake_acls_acl2blob(TALLOC_CTX *mem_ctx, SMB_ACL_T acl)
 {
 	enum ndr_err_code ndr_err;
 	DATA_BLOB blob;
-	ndr_err = ndr_push_struct_blob(&blob, mem_ctx, acl, 
+	ndr_err = ndr_push_struct_blob(&blob, mem_ctx, acl,
 		(ndr_push_flags_fn_t)ndr_push_smb_acl_t);
 
 	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
@@ -306,7 +314,7 @@ static SMB_ACL_T fake_acls_sys_acl_get_fd(struct vfs_handle_struct *handle,
 	const char *name = NULL;
 	struct smb_acl_t *acl = NULL;
 	TALLOC_CTX *frame = talloc_stackframe();
-		
+
 	switch (type) {
 	case SMB_ACL_TYPE_ACCESS:
 		name = FAKE_ACL_ACCESS_XATTR;
@@ -693,7 +701,7 @@ static struct vfs_fn_pointers vfs_fake_acls_fns = {
 	.sys_acl_delete_def_fd_fn = fake_acls_sys_acl_delete_def_fd,
 	.lchown_fn = fake_acls_lchown,
 	.fchown_fn = fake_acls_fchown,
-	
+
 };
 
 static_decl_vfs;
