@@ -19,14 +19,17 @@
 
 
 #include "includes.h"
-#include "popt_common.h"
 #include "utils/net.h"
 #include "rpc_client/cli_pipe.h"
 #include "../librpc/gen_ndr/ndr_samr.h"
 #include "lib/netapi/netapi.h"
 #include "lib/netapi/netapi_net.h"
 #include "../libcli/smbreadline/smbreadline.h"
-#include "libsmb/libsmb.h"
+#include "source3/include/client.h"
+#include "source3/libsmb/proto.h"
+#include "libcli/security/dom_sid.h"
+
+#include <popt.h>
 
 static NTSTATUS rpc_sh_info(struct net_context *c,
 			    TALLOC_CTX *mem_ctx, struct rpc_sh_ctx *ctx,
@@ -50,14 +53,14 @@ static char **completion_fn(const char *text, int start, int end)
 		return NULL;
 	}
 
-	ADD_TO_ARRAY(NULL, char *, SMB_STRDUP(text), &cmds, &n_cmds);
+	ADD_TO_MALLOC_ARRAY(char *, SMB_STRDUP(text), &cmds, &n_cmds);
 
 	for (c = this_ctx->cmds; c->name != NULL; c++) {
 		bool match = (strncmp(text, c->name, strlen(text)) == 0);
 
 		if (match) {
-			ADD_TO_ARRAY(NULL, char *, SMB_STRDUP(c->name),
-				     &cmds, &n_cmds);
+			ADD_TO_MALLOC_ARRAY(char *, SMB_STRDUP(c->name),
+					    &cmds, &n_cmds);
 		}
 	}
 
@@ -67,7 +70,7 @@ static char **completion_fn(const char *text, int start, int end)
 		n_cmds -= 1;
 	}
 
-	ADD_TO_ARRAY(NULL, char *, NULL, &cmds, &n_cmds);
+	ADD_TO_MALLOC_ARRAY(char *, NULL, &cmds, &n_cmds);
 	return cmds;
 }
 
@@ -219,20 +222,18 @@ int net_rpc_shell(struct net_context *c, int argc, const char **argv)
 {
 	NTSTATUS status;
 	struct rpc_sh_ctx *ctx;
+	struct dom_sid_buf buf;
+	NET_API_STATUS net_api_status;
 
 	if (argc != 0 || c->display_usage) {
 		d_printf("%s\nnet rpc shell\n", _("Usage:"));
 		return -1;
 	}
 
-	if (libnetapi_net_init(&c->netapi_ctx) != 0) {
+	net_api_status = libnetapi_net_init(&c->netapi_ctx, c->lp_ctx, c->creds);
+	if (net_api_status != 0) {
 		return -1;
 	}
-	libnetapi_set_username(c->netapi_ctx, c->opt_user_name);
-	libnetapi_set_password(c->netapi_ctx, c->opt_password);
-	if (c->opt_kerberos) {
-		libnetapi_set_use_kerberos(c->netapi_ctx);
-	}
 
 	ctx = talloc(NULL, struct rpc_sh_ctx);
 	if (ctx == NULL) {
@@ -258,7 +259,7 @@ int net_rpc_shell(struct net_context *c, int argc, const char **argv)
 	}
 
 	d_printf(_("Talking to domain %s (%s)\n"), ctx->domain_name,
-		 sid_string_tos(ctx->domain_sid));
+		 dom_sid_str_buf(ctx->domain_sid, &buf));
 
 	this_ctx = ctx;
 
