@@ -4,7 +4,6 @@
 import optparse
 import sys
 import os
-import itertools
 from time import time
 from binascii import hexlify
 
@@ -20,7 +19,7 @@ from samba.samdb import SamDB
 from samba.dcerpc import misc
 from samba import colour
 
-parser = optparse.OptionParser("linked_attributes.py [options] <host>")
+parser = optparse.OptionParser("subtree_rename.py [options] <host>")
 sambaopts = options.SambaOptions(parser)
 parser.add_option_group(sambaopts)
 parser.add_option_group(options.VersionOptions(parser))
@@ -89,7 +88,10 @@ class SubtreeRenameTests(samba.tests.TestCase):
         if not opts.no_cleanup:
             self.delete_ous()
 
-    def add_object(self, cn, objectclass, ou=None, more_attrs={}):
+    def add_object(self, cn, objectclass, ou=None, more_attrs=None):
+        if more_attrs is None:
+            more_attrs = {}
+
         dn = "CN=%s,%s" % (cn, ou)
         attrs = {'cn': cn,
                  'objectclass': objectclass,
@@ -99,7 +101,10 @@ class SubtreeRenameTests(samba.tests.TestCase):
 
         return dn
 
-    def add_objects(self, n, objectclass, prefix=None, ou=None, more_attrs={}):
+    def add_objects(self, n, objectclass, prefix=None, ou=None, more_attrs=None):
+        if more_attrs is None:
+            more_attrs = {}
+
         if prefix is None:
             prefix = objectclass
         dns = []
@@ -200,31 +205,6 @@ class SubtreeRenameTests(samba.tests.TestCase):
                                 attrs=['objectGUID'])
         return str(misc.GUID(res[0]['objectGUID'][0]))
 
-    def assertRaisesLdbError(self, errcode, message, f, *args, **kwargs):
-        """Assert a function raises a particular LdbError."""
-        try:
-            f(*args, **kwargs)
-        except ldb.LdbError as e:
-            (num, msg) = e.args
-            if num != errcode:
-                lut = {v: k for k, v in vars(ldb).items()
-                       if k.startswith('ERR_') and isinstance(v, int)}
-                self.fail("%s, expected "
-                          "LdbError %s, (%d) "
-                          "got %s (%d) "
-                          "%s" % (message,
-                                  lut.get(errcode), errcode,
-                                  lut.get(num), num,
-                                  msg))
-        else:
-            lut = {v: k for k, v in vars(ldb).items()
-                   if k.startswith('ERR_') and isinstance(v, int)}
-            self.fail("%s, expected "
-                      "LdbError %s, (%d) "
-                      "but we got success" % (message,
-                                              lut.get(errcode),
-                                              errcode))
-
     def test_la_move_ou_tree(self):
         tag = 'move_tree'
 
